﻿using DataSharing.Province.SendTask.SubmitCaseResult;
using DataSharing.SendTask;
using DataSharing.Share.Dtos.Province;
using DataSharing.Share.Enums;
using MapsterMapper;
using Microsoft.Extensions.Logging;
using Quartz;
using System.Text.Json;
using XF.Domain.Repository;

namespace DataSharing.Province.SendTask
{
    /// <summary>
    /// 服务工单处理结果     
    /// </summary>
    public class TaskSubmitCaseResultJob : IJob, IDisposable
    {
        private readonly IMapper _mapper;
        private readonly ILogger<TaskSubmitCaseResultJob> _logger;
        private readonly IRepository<DsSendTaskSubmitCaseResult> _sendTaskSubmitCaseResultRepository;
        private readonly IRepository<DsWaitSendTaskSubmitCaseResult> _waitSendTaskSubmitCaseResultRepository;
        private readonly IChannelConfigurationManager _channelConfigurationManager;
        private readonly HuiJuClient _huiJuClient;
        private readonly IRepository<DsSendTaskDetailInfo> _sendTaskDetailInfoRepository;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="mapper"></param>
        /// <param name="logger"></param>
        /// <param name="sendTaskSubmitCaseResultRepository"></param>
        /// <param name="waitSendTaskSubmitCaseResultRepository"></param>
        /// <param name="channelConfigurationManager"></param>
        /// <param name="huiJuClient"></param>
        /// <param name="sendTaskDetailInfoRepository"></param>
        public TaskSubmitCaseResultJob(IMapper mapper, ILogger<TaskSubmitCaseResultJob> logger,
            IRepository<DsSendTaskSubmitCaseResult> sendTaskSubmitCaseResultRepository,
            IRepository<DsWaitSendTaskSubmitCaseResult> waitSendTaskSubmitCaseResultRepository,
            IChannelConfigurationManager channelConfigurationManager,
            HuiJuClient huiJuClient,
           IRepository<DsSendTaskDetailInfo> sendTaskDetailInfoRepository)
        {
            _mapper = mapper;
            _logger = logger;
            _sendTaskSubmitCaseResultRepository = sendTaskSubmitCaseResultRepository;
            _waitSendTaskSubmitCaseResultRepository = waitSendTaskSubmitCaseResultRepository;
            _channelConfigurationManager = channelConfigurationManager;
            _huiJuClient = huiJuClient;
            _sendTaskDetailInfoRepository = sendTaskDetailInfoRepository;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context"></param>
        /// <returns></returns>
        public async Task Execute(IJobExecutionContext context)
        {
            var configProvince = _channelConfigurationManager.GetConfigurationProvince();
            //加载数据
            var tasks = await _waitSendTaskSubmitCaseResultRepository.Queryable()
           .Where(d => d.State == EWaitSendTaskState.WaitPush && d.SendTimes <= 6)
           .OrderBy(d => d.CreationTime)
           .Take(100)
           .ToListAsync(context.CancellationToken);

            if (tasks.Count != 0)
            {
                #region 处理请求参数
                var request = new ProvinceListRequest<Share.Dtos.Province.HuiJu.SubmitCaseResultInfo>(configProvince.ClientId, configProvince.ClientSecret);
                List<Share.Dtos.Province.HuiJu.SubmitCaseResultInfo> listOrder = [];
                #endregion

                List<DsWaitSendTaskSubmitCaseResult> taskData = [];

                //处理推送数据
                foreach (var sendTask in tasks)
                {
                    //状态更改为推送中
                    sendTask.State = EWaitSendTaskState.Pushing;
                    //使用乐观锁，控制数据
                    if (await _waitSendTaskSubmitCaseResultRepository.Updateable(sendTask).ExecuteCommandWithOptLockAsync() > 0)
                    {
                        //组装这一条数据
                        if (sendTask.RequestData != null)
                        {
                            listOrder.Add(sendTask.RequestData);
                            taskData.Add(sendTask);
                        }
                    }
                }

                if (listOrder != null && listOrder.Count > 0)
                {
                    request.SetData(listOrder);
                    var jsonData = request.ToString();
                    ProvinceResponse result = new();
                    string error = "";
                    //执行推送任务
                    try
                    {
                        result = await _huiJuClient.RequestAsync<ProvinceResponse>(configProvince.HuiJu + PtahDefaults.SubmitCaseResult, "Post", jsonData, context.CancellationToken);
                    }
                    catch (Exception ex)
                    {
                        error = ex.Message;
                    }

                    if (result != null)
                    {
                        //处理数据
                        foreach (var item in taskData)
                        {
                            //查询是否有错误
                            //有错误处理
                            //没有错误删除待推送表，更新主表
                            bool isError = false;
                            if (result == null || result.Custom == null || result.ReturnInfo == null)
                            {
                                //请求报错，那么则所有的推送失败
                                isError = true;
                            }
                            else
                            {
                                if (result.Custom.ErrorObjs.Exists(p => p.CaseSerial == item.ProvinceNo))
                                    isError = true;
                            }

                            //处理待推送数据
                            if (isError)
                            {
                                //修改
                                item.LastTime = DateTime.Now;
                                item.SendTimes = item.SendTimes + 1;

                                if (item.FirstTime is null)
                                    item.FirstTime = DateTime.Now;

                                if (item.SendTimes >= 7)
                                    item.State = EWaitSendTaskState.PushFail;

                                //修改待推送表
                                await _waitSendTaskSubmitCaseResultRepository.UpdateAsync(item, context.CancellationToken);
                            }
                            else
                            {
                                //删除
                                await _waitSendTaskSubmitCaseResultRepository.RemoveAsync(item, cancellationToken: context.CancellationToken);
                            }

                            //查询主表
                            var sendData = await _sendTaskSubmitCaseResultRepository.GetAsync(p => p.Id == item.Id, context.CancellationToken);
                            if (sendData != null)
                            {
                                sendData.LastTime = DateTime.Now;
                                sendData.SendTimes = sendData.SendTimes + 1;

                                if (sendData.FirstTime is null)
                                    sendData.FirstTime = DateTime.Now;

                                sendData.IsSuccess = isError == false ? ESendTaskState.PushSuccess : ESendTaskState.PushFail;
                                await _sendTaskSubmitCaseResultRepository.UpdateAsync(sendData, cancellationToken: context.CancellationToken);
                            }

                            //新增明细
                            DsSendTaskDetailInfo dsSendTaskDetailInfo = new()
                            {
                                TaskId = item.Id,
                                IsSuccess = !isError,
                                Result = JsonSerializer.Serialize(result),
                                ResultErrorData = error,
                            };
                            await _sendTaskDetailInfoRepository.AddAsync(dsSendTaskDetailInfo, context.CancellationToken);
                        }
                    }
                }
            }
        }



        public void Dispose()
        {

        }
    }
}